@@ -6,6 +6,7 @@ import org.bouncycastle.asn1.ASN1TaggedObject;
 import org.bouncycastle.asn1.DEREncodable;
 import org.bouncycastle.asn1.DERObject;
 import org.bouncycastle.asn1.DERObjectIdentifier;
+import org.bouncycastle.asn1.eac.EACObjectIdentifiers;
 import org.bouncycastle.asn1.cryptopro.CryptoProObjectIdentifiers;
 import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
 import org.bouncycastle.asn1.oiw.OIWObjectIdentifiers;
@@ -25,6 +26,7 @@ import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.security.NoSuchProviderException;
 import java.security.Signature;
+import java.security.Provider;
 import java.security.cert.CRLException;
 import org.bouncycastle.jce.cert.CertStore;
 import java.security.cert.CertificateException;
@@ -75,6 +77,15 @@ class CMSSignedHelper
         addEntries(X9ObjectIdentifiers.ecdsa_with_SHA384, "SHA384", "ECDSA");
         addEntries(X9ObjectIdentifiers.ecdsa_with_SHA512, "SHA512", "ECDSA");
         addEntries(X9ObjectIdentifiers.id_dsa_with_sha1, "SHA1", "DSA");
+        addEntries(EACObjectIdentifiers.id_TA_ECDSA_SHA_1, "SHA1", "ECDSA");
+        addEntries(EACObjectIdentifiers.id_TA_ECDSA_SHA_224, "SHA224", "ECDSA");
+        addEntries(EACObjectIdentifiers.id_TA_ECDSA_SHA_256, "SHA256", "ECDSA");
+        addEntries(EACObjectIdentifiers.id_TA_ECDSA_SHA_384, "SHA384", "ECDSA");
+        addEntries(EACObjectIdentifiers.id_TA_ECDSA_SHA_512, "SHA512", "ECDSA");
+        addEntries(EACObjectIdentifiers.id_TA_RSA_v1_5_SHA_1, "SHA1", "RSA");
+        addEntries(EACObjectIdentifiers.id_TA_RSA_v1_5_SHA_256, "SHA256", "RSA");
+        addEntries(EACObjectIdentifiers.id_TA_RSA_PSS_SHA_1, "SHA1", "RSAandMGF1");
+        addEntries(EACObjectIdentifiers.id_TA_RSA_PSS_SHA_256, "SHA256", "RSAandMGF1");
 
         encryptionAlgs.put(X9ObjectIdentifiers.id_dsa.getId(), "DSA");
         encryptionAlgs.put(PKCSObjectIdentifiers.rsaEncryption.getId(), "RSA");
@@ -157,8 +168,8 @@ class CMSSignedHelper
     
     MessageDigest getDigestInstance(
         String algorithm, 
-        String provider) 
-        throws NoSuchProviderException, NoSuchAlgorithmException
+        Provider provider)
+        throws NoSuchAlgorithmException, NoSuchProviderException
     {
         try
         {
@@ -188,12 +199,12 @@ class CMSSignedHelper
 
     private MessageDigest createDigestInstance(
         String algorithm,
-        String provider)
+        Provider provider)
         throws NoSuchAlgorithmException, NoSuchProviderException
     {
         if (provider != null)
         {
-            return MessageDigest.getInstance(algorithm, provider);
+            return MessageDigest.getInstance(algorithm, provider.getName());
         }
         else
         {
@@ -203,12 +214,12 @@ class CMSSignedHelper
 
     Signature getSignatureInstance(
         String algorithm, 
-        String provider) 
-        throws NoSuchProviderException, NoSuchAlgorithmException
+        Provider provider)
+        throws NoSuchAlgorithmException, NoSuchProviderException
     {
         if (provider != null)
         {
-            return Signature.getInstance(algorithm, provider);
+            return Signature.getInstance(algorithm, provider.getName());
         }
         else
         {
@@ -218,9 +229,9 @@ class CMSSignedHelper
 
     X509Store createAttributeStore(
         String type,
-        String provider,
+        Provider provider,
         ASN1Set certSet)
-        throws NoSuchStoreException, NoSuchProviderException, CMSException
+        throws NoSuchStoreException, CMSException
     {
         List certs = new ArrayList();
 
@@ -265,9 +276,9 @@ class CMSSignedHelper
 
     X509Store createCertificateStore(
         String type,
-        String provider,
+        Provider provider,
         ASN1Set certSet)
-        throws NoSuchStoreException, NoSuchProviderException, CMSException
+        throws NoSuchStoreException, CMSException
     {
         List certs = new ArrayList();
 
@@ -289,9 +300,9 @@ class CMSSignedHelper
 
     X509Store createCRLsStore(
         String type,
-        String provider,
+        Provider provider,
         ASN1Set crlSet)
-        throws NoSuchStoreException, NoSuchProviderException, CMSException
+        throws NoSuchStoreException, CMSException
     {
         List crls = new ArrayList();
 
@@ -313,10 +324,10 @@ class CMSSignedHelper
 
     CertStore createCertStore(
         String type,
-        String provider,
+        Provider provider,
         ASN1Set certSet,
         ASN1Set crlSet)
-        throws NoSuchProviderException, CMSException, NoSuchAlgorithmException
+        throws CMSException, NoSuchAlgorithmException
     {
         List certsAndcrls = new ArrayList();
 
@@ -338,21 +349,25 @@ class CMSSignedHelper
         {
             if (provider != null)
             {
-                return CertStore.getInstance(type, new CollectionCertStoreParameters(certsAndcrls), provider);
+                return CertStore.getInstance(type, new CollectionCertStoreParameters(certsAndcrls), provider.getName());
             }
             else
             {
                 return CertStore.getInstance(type, new CollectionCertStoreParameters(certsAndcrls));
             }
         }
+        catch (NoSuchProviderException ex)
+        {
+            throw new CMSException("can't find provider.", ex);
+        }
         catch (InvalidAlgorithmParameterException e)
         {
             throw new CMSException("can't setup the CertStore", e);
         }
     }
 
-    private void addCertsFromSet(List certs, ASN1Set certSet, String provider)
-        throws NoSuchProviderException, CMSException
+    private void addCertsFromSet(List certs, ASN1Set certSet, Provider provider)
+        throws CMSException
     {
         CertificateFactory cf;
 
@@ -360,13 +375,17 @@ class CMSSignedHelper
         {
             if (provider != null)
             {
-                cf = CertificateFactory.getInstance("X.509", provider);
+                cf = CertificateFactory.getInstance("X.509", provider.getName());
             }
             else
             {
                 cf = CertificateFactory.getInstance("X.509");
             }
         }
+        catch (NoSuchProviderException ex)
+        {
+            throw new CMSException("can't find provider.", ex);
+        }
         catch (CertificateException ex)
         {
             throw new CMSException("can't get certificate factory.", ex);
@@ -398,8 +417,8 @@ class CMSSignedHelper
         }
     }
 
-    private void addCRLsFromSet(List crls, ASN1Set certSet, String provider)
-        throws NoSuchProviderException, CMSException
+    private void addCRLsFromSet(List crls, ASN1Set certSet, Provider provider)
+        throws CMSException
     {
         CertificateFactory cf;
 
@@ -407,13 +426,17 @@ class CMSSignedHelper
         {
             if (provider != null)
             {
-                cf = CertificateFactory.getInstance("X.509", provider);
+                cf = CertificateFactory.getInstance("X.509", provider.getName());
             }
             else
             {
                 cf = CertificateFactory.getInstance("X.509");
             }
         }
+        catch (NoSuchProviderException ex)
+        {
+            throw new CMSException("can't find provider.", ex);
+        }
         catch (CertificateException ex)
         {
             throw new CMSException("can't get certificate factory.", ex);
