@@ -57,4 +57,16 @@ public class LiteralDataPacket
     {
         return new String(fileName);
     }
+
+    public byte[] getRawFileName()
+    {
+        byte[] tmp = new byte[fileName.length];
+
+        for (int i = 0; i != tmp.length; i++)
+        {
+            tmp[i] = (byte)fileName[i];
+        }
+
+        return tmp;
+    }
 }
