@@ -32,6 +32,7 @@ import java.security.NoSuchProviderException;
 import java.security.PublicKey;
 import java.security.Signature;
 import java.security.SignatureException;
+import java.security.Provider;
 import java.security.cert.CertificateExpiredException;
 import java.security.cert.CertificateNotYetValidException;
 import java.security.cert.X509Certificate;
@@ -77,7 +78,7 @@ public class SignerInformation
             {
                 ASN1OctetString octs = ASN1OctetString.getInstance(s.getId());
 
-                sid.setSubjectKeyIdentifier(octs.getOctets());
+                sid.setSubjectKeyIdentifier(octs.getEncoded());
             }
             else
             {
@@ -172,7 +173,7 @@ public class SignerInformation
     }
 
     /**
-     * return the signature/encyrption algorithm parameters, or null if
+     * return the signature/encryption algorithm parameters, or null if
      * there aren't any.
      */
     public byte[] getEncryptionAlgParams()
@@ -229,22 +230,59 @@ public class SignerInformation
      */
     public SignerInformationStore getCounterSignatures()
     {
+        // TODO There are several checks implied by the RFC3852 comments that are missing
+
+        /*
+        The countersignature attribute MUST be an unsigned attribute; it MUST
+        NOT be a signed attribute, an authenticated attribute, an
+        unauthenticated attribute, or an unprotected attribute.
+        */        
         AttributeTable unsignedAttributeTable = getUnsignedAttributes();
         if (unsignedAttributeTable == null)
         {
             return new SignerInformationStore(new ArrayList(0));
         }
-        
+
         List counterSignatures = new ArrayList();
 
-        Attribute counterSignatureAttribute = unsignedAttributeTable.get(CMSAttributes.counterSignature);
-        if (counterSignatureAttribute != null)
+        /*
+        The UnsignedAttributes syntax is defined as a SET OF Attributes.  The
+        UnsignedAttributes in a signerInfo may include multiple instances of
+        the countersignature attribute.
+        */
+        ASN1EncodableVector allCSAttrs = unsignedAttributeTable.getAll(CMSAttributes.counterSignature);
+
+        for (int i = 0; i < allCSAttrs.size(); ++i)
         {
+            Attribute counterSignatureAttribute = (Attribute)allCSAttrs.get(i);            
+
+            /*
+            A countersignature attribute can have multiple attribute values.  The
+            syntax is defined as a SET OF AttributeValue, and there MUST be one
+            or more instances of AttributeValue present.
+            */
             ASN1Set values = counterSignatureAttribute.getAttrValues();
-            counterSignatures = new ArrayList(values.size());
+            if (values.size() < 1)
+            {
+                // TODO Throw an appropriate exception?
+            }
 
             for (Enumeration en = values.getObjects(); en.hasMoreElements();)
             {
+                /*
+                Countersignature values have the same meaning as SignerInfo values
+                for ordinary signatures, except that:
+
+                   1. The signedAttributes field MUST NOT contain a content-type
+                      attribute; there is no content type for countersignatures.
+
+                   2. The signedAttributes field MUST contain a message-digest
+                      attribute if it contains any other attributes.
+
+                   3. The input to the message-digesting process is the contents
+                      octets of the DER encoding of the signatureValue field of the
+                      SignerInfo value with which the attribute is associated.
+                */
                 SignerInfo si = SignerInfo.getInstance(en.nextElement());
 
                 String          digestName = CMSSignedHelper.INSTANCE.getDigestAlgName(si.getDigestAlgorithm().getObjectId().getId());
@@ -274,14 +312,61 @@ public class SignerInformation
     private boolean doVerify(
         PublicKey       key,
         AttributeTable  signedAttrTable,
-        String          sigProvider)
-        throws CMSException, NoSuchAlgorithmException, NoSuchProviderException
+        Provider        sigProvider)
+        throws CMSException, NoSuchAlgorithmException
     {
         String          digestName = CMSSignedHelper.INSTANCE.getDigestAlgName(this.getDigestAlgOID());
         String          signatureName = digestName + "with" + CMSSignedHelper.INSTANCE.getEncryptionAlgName(this.getEncryptionAlgOID());
         Signature       sig = CMSSignedHelper.INSTANCE.getSignatureInstance(signatureName, sigProvider);
         MessageDigest   digest = CMSSignedHelper.INSTANCE.getDigestInstance(digestName, sigProvider); 
-        
+
+        // TODO [BJA-109] Note: PSSParameterSpec requires JDK1.4+ 
+/*
+        try
+        {
+            DERObjectIdentifier sigAlgOID = encryptionAlgorithm.getObjectId();
+            DEREncodable sigParams = this.encryptionAlgorithm.getParameters();
+            if (sigAlgOID.equals(PKCSObjectIdentifiers.id_RSASSA_PSS))
+            {
+                // RFC 4056
+                // When the id-RSASSA-PSS algorithm identifier is used for a signature,
+                // the AlgorithmIdentifier parameters field MUST contain RSASSA-PSS-params.
+                if (sigParams == null)
+                {
+                    throw new CMSException(
+                        "RSASSA-PSS signature must specify algorithm parameters");
+                }
+
+                AlgorithmParameters params = AlgorithmParameters.getInstance(
+                    sigAlgOID.getId(), sig.getProvider().getName());
+                params.init(sigParams.getDERObject().getEncoded(), "ASN.1");
+
+                PSSParameterSpec spec = (PSSParameterSpec)params.getParameterSpec(PSSParameterSpec.class);
+                sig.setParameter(spec);
+            }
+            else
+            {
+                // TODO Are there other signature algorithms that provide parameters?
+                if (sigParams != null)
+                {
+                    throw new CMSException("unrecognised signature parameters provided");
+                }
+            }
+        }
+        catch (IOException e)
+        {
+            throw new CMSException("error encoding signature parameters.", e);
+        }
+        catch (InvalidAlgorithmParameterException e)
+        {
+            throw new CMSException("error setting signature parameters.", e);
+        }
+        catch (InvalidParameterSpecException e)
+        {
+            throw new CMSException("error processing signature parameters.", e);
+        }
+*/
+
         try
         {
             sig.initVerify(key);
@@ -426,8 +511,8 @@ public class SignerInformation
         byte[]    digest, 
         PublicKey key,
         byte[]    signature,
-        String    sigProvider) 
-        throws NoSuchAlgorithmException, NoSuchProviderException, CMSException
+        Provider  sigProvider)
+        throws NoSuchAlgorithmException, CMSException
     {
         String algorithm = CMSSignedHelper.INSTANCE.getEncryptionAlgName(this.getEncryptionAlgOID());
         
@@ -505,7 +590,19 @@ public class SignerInformation
         String      sigProvider)
         throws NoSuchAlgorithmException, NoSuchProviderException, CMSException
     {
-        return doVerify(key, this.getSignedAttributes(), sigProvider); 
+        return doVerify(key, this.getSignedAttributes(), CMSUtils.getProvider(sigProvider));
+    }
+
+    /**
+     * verify that the given public key succesfully handles and confirms the
+     * signature associated with this signer.
+     */
+    public boolean verify(
+        PublicKey   key,
+        Provider    sigProvider)
+        throws NoSuchAlgorithmException, NoSuchProviderException, CMSException
+    {
+        return doVerify(key, this.getSignedAttributes(), sigProvider);
     }
 
     /**
@@ -520,6 +617,22 @@ public class SignerInformation
         throws NoSuchAlgorithmException, NoSuchProviderException,
             CertificateExpiredException, CertificateNotYetValidException,
             CMSException
+    {
+        return verify(cert, CMSUtils.getProvider(sigProvider));
+    }
+
+    /**
+     * verify that the given certificate successfully handles and confirms
+     * the signature associated with this signer and, if a signingTime
+     * attribute is available, that the certificate was valid at the time the
+     * signature was generated.
+     */
+    public boolean verify(
+        X509Certificate cert,
+        Provider        sigProvider)
+        throws NoSuchAlgorithmException,
+            CertificateExpiredException, CertificateNotYetValidException,
+            CMSException
     {
         AttributeTable attr = this.getSignedAttributes();
 
