@@ -1,7 +1,7 @@
 package org.bouncycastle.jce.provider;
 
+import org.bouncycastle.asn1.ASN1Encodable;
 import org.bouncycastle.asn1.ASN1EncodableVector;
-import org.bouncycastle.asn1.ASN1InputStream;
 import org.bouncycastle.asn1.ASN1Object;
 import org.bouncycastle.asn1.ASN1OctetString;
 import org.bouncycastle.asn1.ASN1Sequence;
@@ -9,10 +9,8 @@ import org.bouncycastle.asn1.DERInteger;
 import org.bouncycastle.asn1.DERNull;
 import org.bouncycastle.asn1.DERObjectIdentifier;
 import org.bouncycastle.asn1.DEROctetString;
-import org.bouncycastle.asn1.DEROutputStream;
 import org.bouncycastle.asn1.DERSequence;
 import org.bouncycastle.asn1.cryptopro.GOST3410PublicKeyAlgParameters;
-import org.bouncycastle.asn1.misc.IDEACBCPar;
 import org.bouncycastle.asn1.oiw.ElGamalParameter;
 import org.bouncycastle.asn1.pkcs.DHParameter;
 import org.bouncycastle.asn1.pkcs.PKCS12PBEParams;
@@ -20,12 +18,14 @@ import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.RC2CBCParameter;
 import org.bouncycastle.asn1.pkcs.RSAESOAEPparams;
 import org.bouncycastle.asn1.pkcs.RSASSAPSSparams;
+import org.bouncycastle.asn1.pkcs.PBKDF2Params;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
 import org.bouncycastle.asn1.x509.DSAParameter;
 import org.bouncycastle.jce.spec.ElGamalParameterSpec;
 import org.bouncycastle.jce.spec.GOST3410ParameterSpec;
 import org.bouncycastle.jce.spec.GOST3410PublicKeyParameterSetSpec;
 import org.bouncycastle.jce.spec.IESParameterSpec;
+import org.bouncycastle.util.Arrays;
 
 import javax.crypto.spec.DHParameterSpec;
 import javax.crypto.spec.IvParameterSpec;
@@ -33,7 +33,6 @@ import javax.crypto.spec.OAEPParameterSpec;
 import javax.crypto.spec.PBEParameterSpec;
 import javax.crypto.spec.PSource;
 import javax.crypto.spec.RC2ParameterSpec;
-import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.security.AlgorithmParametersSpi;
 import java.security.spec.AlgorithmParameterSpec;
@@ -87,10 +86,7 @@ public abstract class JDKAlgorithmParameters
             
             if (format.equals("RAW"))
             {
-                byte[]  tmp = new byte[iv.length];
-
-                System.arraycopy(iv, 0, tmp, 0, iv.length);
-                return tmp;
+                return Arrays.clone(iv);
             }
 
             return null;
@@ -130,15 +126,12 @@ public abstract class JDKAlgorithmParameters
             if ((params.length % 8) != 0
                     && params[0] == 0x04 && params[1] == params.length - 2)
             {
-                ASN1InputStream         aIn = new ASN1InputStream(params);
-                ASN1OctetString         oct = (ASN1OctetString)aIn.readObject();
+                ASN1OctetString oct = (ASN1OctetString)ASN1Object.fromByteArray(params);
 
                 params = oct.getOctets();
             }
 
-            this.iv = new byte[params.length];
-
-            System.arraycopy(params, 0, iv, 0, iv.length);
+            this.iv = Arrays.clone(params);
         }
 
         protected void engineInit(
@@ -148,12 +141,10 @@ public abstract class JDKAlgorithmParameters
         {
             if (isASN1FormatString(format))
             {
-                ASN1InputStream         aIn = new ASN1InputStream(params);
-                
                 try
                 {
-                    ASN1OctetString         oct = (ASN1OctetString)aIn.readObject();
-    
+                    ASN1OctetString oct = (ASN1OctetString)ASN1Object.fromByteArray(params);
+
                     engineInit(oct.getOctets());
                 }
                 catch (Exception e)
@@ -179,98 +170,6 @@ public abstract class JDKAlgorithmParameters
         }
     }
 
-    public static class IDEAAlgorithmParameters
-        extends JDKAlgorithmParameters
-    {
-        private byte[]  iv;
-
-        protected byte[] engineGetEncoded() 
-            throws IOException
-        {
-            return engineGetEncoded("ASN.1");
-        }
-
-        protected byte[] engineGetEncoded(
-            String format) 
-            throws IOException
-        {
-            if (isASN1FormatString(format))
-            {
-                return new IDEACBCPar(engineGetEncoded("RAW")).getEncoded();
-            }
-            
-            if (format.equals("RAW"))
-            {
-                byte[]  tmp = new byte[iv.length];
-
-                System.arraycopy(iv, 0, tmp, 0, iv.length);
-                return tmp;
-            }
-
-            return null;
-        }
-
-        protected AlgorithmParameterSpec localEngineGetParameterSpec(
-            Class paramSpec) 
-            throws InvalidParameterSpecException
-        {
-            if (paramSpec == IvParameterSpec.class)
-            {
-                return new IvParameterSpec(iv);
-            }
-
-            throw new InvalidParameterSpecException("unknown parameter spec passed to IV parameters object.");
-        }
-
-        protected void engineInit(
-            AlgorithmParameterSpec paramSpec) 
-            throws InvalidParameterSpecException
-        {
-            if (!(paramSpec instanceof IvParameterSpec))
-            {
-                throw new InvalidParameterSpecException("IvParameterSpec required to initialise a IV parameters algorithm parameters object");
-            }
-
-            this.iv = ((IvParameterSpec)paramSpec).getIV();
-        }
-
-        protected void engineInit(
-            byte[] params) 
-            throws IOException
-        {
-            this.iv = new byte[params.length];
-
-            System.arraycopy(params, 0, iv, 0, iv.length);
-        }
-
-        protected void engineInit(
-            byte[] params,
-            String format) 
-            throws IOException
-        {
-            if (format.equals("RAW"))
-            {
-                engineInit(params);
-                return;
-            }
-            if (format.equals("ASN.1"))
-            {
-                ASN1InputStream         aIn = new ASN1InputStream(params);
-                IDEACBCPar              oct = new IDEACBCPar((ASN1Sequence)aIn.readObject());
-
-                engineInit(oct.getIV());
-                return;
-            }
-
-            throw new IOException("Unknown parameters format in IV parameters object");
-        }
-
-        protected String engineToString() 
-        {
-            return "IDEA Parameters";
-        }
-    }
-
     public static class RC2AlgorithmParameters
         extends JDKAlgorithmParameters
     {
@@ -317,10 +216,7 @@ public abstract class JDKAlgorithmParameters
 
         protected byte[] engineGetEncoded() 
         {
-            byte[]  tmp = new byte[iv.length];
-
-            System.arraycopy(iv, 0, tmp, 0, iv.length);
-            return tmp;
+            return Arrays.clone(iv);
         }
 
         protected byte[] engineGetEncoded(
@@ -409,9 +305,7 @@ public abstract class JDKAlgorithmParameters
             byte[] params) 
             throws IOException
         {
-            this.iv = new byte[params.length];
-
-            System.arraycopy(params, 0, iv, 0, iv.length);
+            this.iv = Arrays.clone(params);
         }
 
         protected void engineInit(
@@ -421,8 +315,7 @@ public abstract class JDKAlgorithmParameters
         {
             if (isASN1FormatString(format))
             {
-                ASN1InputStream         aIn = new ASN1InputStream(params);
-                RC2CBCParameter         p = RC2CBCParameter.getInstance(aIn.readObject());
+                RC2CBCParameter p = RC2CBCParameter.getInstance(ASN1Object.fromByteArray(params));
 
                 if (p.getRC2ParameterVersion() != null)
                 {
@@ -449,6 +342,89 @@ public abstract class JDKAlgorithmParameters
         }
     }
 
+    public static class PBKDF2
+        extends JDKAlgorithmParameters
+    {
+        PBKDF2Params params;
+
+        protected byte[] engineGetEncoded()
+        {
+            try
+            {
+                return params.getEncoded(ASN1Encodable.DER);
+            }
+            catch (IOException e)
+            {
+                throw new RuntimeException("Oooops! " + e.toString());
+            }
+        }
+
+        protected byte[] engineGetEncoded(
+            String format)
+        {
+            if (isASN1FormatString(format))
+            {
+                return engineGetEncoded();
+            }
+
+            return null;
+        }
+
+        protected AlgorithmParameterSpec localEngineGetParameterSpec(
+            Class paramSpec)
+            throws InvalidParameterSpecException
+        {
+            if (paramSpec == PBEParameterSpec.class)
+            {
+                return new PBEParameterSpec(params.getSalt(),
+                                params.getIterationCount().intValue());
+            }
+
+            throw new InvalidParameterSpecException("unknown parameter spec passed to PKCS12 PBE parameters object.");
+        }
+
+        protected void engineInit(
+            AlgorithmParameterSpec paramSpec)
+            throws InvalidParameterSpecException
+        {
+            if (!(paramSpec instanceof PBEParameterSpec))
+            {
+                throw new InvalidParameterSpecException("PBEParameterSpec required to initialise a PKCS12 PBE parameters algorithm parameters object");
+            }
+
+            PBEParameterSpec    pbeSpec = (PBEParameterSpec)paramSpec;
+
+            this.params = new PBKDF2Params(pbeSpec.getSalt(),
+                                pbeSpec.getIterationCount());
+        }
+
+        protected void engineInit(
+            byte[] params)
+            throws IOException
+        {
+            this.params = PBKDF2Params.getInstance(ASN1Object.fromByteArray(params));
+        }
+
+        protected void engineInit(
+            byte[] params,
+            String format)
+            throws IOException
+        {
+            if (isASN1FormatString(format))
+            {
+                engineInit(params);
+                return;
+            }
+
+            throw new IOException("Unknown parameters format in PWRIKEK parameters object");
+        }
+
+        protected String engineToString()
+        {
+            return "PBKDF2 Parameters";
+        }
+    }
+
     public static class PKCS12PBE
         extends JDKAlgorithmParameters
     {
@@ -456,19 +432,14 @@ public abstract class JDKAlgorithmParameters
 
         protected byte[] engineGetEncoded() 
         {
-            ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
-            DEROutputStream         dOut = new DEROutputStream(bOut);
-
             try
             {
-                dOut.writeObject(params);
+                return params.getEncoded(ASN1Encodable.DER);
             }
             catch (IOException e)
             {
                 throw new RuntimeException("Oooops! " + e.toString());
             }
-
-            return bOut.toByteArray();
         }
 
         protected byte[] engineGetEncoded(
@@ -514,9 +485,7 @@ public abstract class JDKAlgorithmParameters
             byte[] params) 
             throws IOException
         {
-            ASN1InputStream        aIn = new ASN1InputStream(params);
-
-            this.params = PKCS12PBEParams.getInstance(aIn.readObject());
+            this.params = PKCS12PBEParams.getInstance(ASN1Object.fromByteArray(params));
         }
 
         protected void engineInit(
@@ -556,21 +525,16 @@ public abstract class JDKAlgorithmParameters
          */
         protected byte[] engineGetEncoded() 
         {
-            ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
-            DEROutputStream         dOut = new DEROutputStream(bOut);
-            DHParameter             dhP = new DHParameter(currentSpec.getP(), currentSpec.getG(), currentSpec.getL());
+            DHParameter dhP = new DHParameter(currentSpec.getP(), currentSpec.getG(), currentSpec.getL());
 
             try
             {
-                dOut.writeObject(dhP);
-                dOut.close();
+                return dhP.getEncoded(ASN1Encodable.DER);                
             }
             catch (IOException e)
             {
                 throw new RuntimeException("Error encoding DHParameters");
             }
-
-            return bOut.toByteArray();
         }
 
         protected byte[] engineGetEncoded(
@@ -612,11 +576,9 @@ public abstract class JDKAlgorithmParameters
             byte[] params) 
             throws IOException
         {
-            ASN1InputStream        aIn = new ASN1InputStream(params);
-
             try
             {
-                DHParameter dhP = new DHParameter((ASN1Sequence)aIn.readObject());
+                DHParameter dhP = new DHParameter((ASN1Sequence)ASN1Object.fromByteArray(params));
 
                 if (dhP.getL() != null)
                 {
@@ -675,21 +637,16 @@ public abstract class JDKAlgorithmParameters
          */
         protected byte[] engineGetEncoded() 
         {
-            ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
-            DEROutputStream         dOut = new DEROutputStream(bOut);
-            DSAParameter            dsaP = new DSAParameter(currentSpec.getP(), currentSpec.getQ(), currentSpec.getG());
+            DSAParameter dsaP = new DSAParameter(currentSpec.getP(), currentSpec.getQ(), currentSpec.getG());
 
             try
             {
-                dOut.writeObject(dsaP);
-                dOut.close();
+                return dsaP.getEncoded(ASN1Encodable.DER);
             }
             catch (IOException e)
             {
                 throw new RuntimeException("Error encoding DSAParameters");
             }
-
-            return bOut.toByteArray();
         }
 
         protected byte[] engineGetEncoded(
@@ -731,11 +688,9 @@ public abstract class JDKAlgorithmParameters
             byte[] params) 
             throws IOException
         {
-            ASN1InputStream        aIn = new ASN1InputStream(params);
-
             try
             {
-                DSAParameter dsaP = new DSAParameter((ASN1Sequence)aIn.readObject());
+                DSAParameter dsaP = new DSAParameter((ASN1Sequence)ASN1Object.fromByteArray(params));
 
                 currentSpec = new DSAParameterSpec(dsaP.getP(), dsaP.getQ(), dsaP.getG());
             }
@@ -787,21 +742,16 @@ public abstract class JDKAlgorithmParameters
          */
         protected byte[] engineGetEncoded()
         {
-            ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
-            DEROutputStream         dOut = new DEROutputStream(bOut);
-            GOST3410PublicKeyAlgParameters       gost3410P = new GOST3410PublicKeyAlgParameters(new DERObjectIdentifier(currentSpec.getPublicKeyParamSetOID()), new DERObjectIdentifier(currentSpec.getDigestParamSetOID()), new DERObjectIdentifier(currentSpec.getEncryptionParamSetOID()));
-            
+            GOST3410PublicKeyAlgParameters gost3410P = new GOST3410PublicKeyAlgParameters(new DERObjectIdentifier(currentSpec.getPublicKeyParamSetOID()), new DERObjectIdentifier(currentSpec.getDigestParamSetOID()), new DERObjectIdentifier(currentSpec.getEncryptionParamSetOID()));
+
             try
             {
-                dOut.writeObject(gost3410P);
-                dOut.close();
+                return gost3410P.getEncoded(ASN1Encodable.DER);
             }
             catch (IOException e)
             {
                 throw new RuntimeException("Error encoding GOST3410Parameters");
             }
-            
-            return bOut.toByteArray();
         }
         
         protected byte[] engineGetEncoded(
@@ -897,21 +847,16 @@ public abstract class JDKAlgorithmParameters
          */
         protected byte[] engineGetEncoded() 
         {
-            ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
-            DEROutputStream         dOut = new DEROutputStream(bOut);
-            ElGamalParameter        elP = new ElGamalParameter(currentSpec.getP(), currentSpec.getG());
+            ElGamalParameter elP = new ElGamalParameter(currentSpec.getP(), currentSpec.getG());
 
             try
             {
-                dOut.writeObject(elP);
-                dOut.close();
+                return elP.getEncoded(ASN1Encodable.DER);
             }
             catch (IOException e)
             {
                 throw new RuntimeException("Error encoding ElGamalParameters");
             }
-
-            return bOut.toByteArray();
         }
 
         protected byte[] engineGetEncoded(
@@ -966,11 +911,9 @@ public abstract class JDKAlgorithmParameters
             byte[] params) 
             throws IOException
         {
-            ASN1InputStream        aIn = new ASN1InputStream(params);
-
             try
             {
-                ElGamalParameter elP = new ElGamalParameter((ASN1Sequence)aIn.readObject());
+                ElGamalParameter elP = new ElGamalParameter((ASN1Sequence)ASN1Object.fromByteArray(params));
 
                 currentSpec = new ElGamalParameterSpec(elP.getP(), elP.getG());
             }
@@ -1011,31 +954,25 @@ public abstract class JDKAlgorithmParameters
         IESParameterSpec     currentSpec;
 
         /**
-         * in the abscence of a standard way of doing it this will do for
+         * in the absence of a standard way of doing it this will do for
          * now...
          */
         protected byte[] engineGetEncoded() 
         {
-            ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
-            DEROutputStream         dOut = new DEROutputStream(bOut);
-
             try
             {
-                ASN1EncodableVector    v = new ASN1EncodableVector();
+                ASN1EncodableVector v = new ASN1EncodableVector();
 
                 v.add(new DEROctetString(currentSpec.getDerivationV()));
                 v.add(new DEROctetString(currentSpec.getEncodingV()));
                 v.add(new DERInteger(currentSpec.getMacKeySize()));
 
-                dOut.writeObject(new DERSequence(v));
-                dOut.close();
+                return new DERSequence(v).getEncoded(ASN1Encodable.DER);
             }
             catch (IOException e)
             {
                 throw new RuntimeException("Error encoding IESParameters");
             }
-
-            return bOut.toByteArray();
         }
 
         protected byte[] engineGetEncoded(
@@ -1077,11 +1014,9 @@ public abstract class JDKAlgorithmParameters
             byte[] params) 
             throws IOException
         {
-            ASN1InputStream        aIn = new ASN1InputStream(params);
-
             try
             {
-                ASN1Sequence    s = (ASN1Sequence)aIn.readObject();
+                ASN1Sequence s = (ASN1Sequence)ASN1Object.fromByteArray(params);
 
                 this.currentSpec = new IESParameterSpec(
                                         ((ASN1OctetString)s.getObjectAt(0)).getOctets(),
@@ -1125,12 +1060,10 @@ public abstract class JDKAlgorithmParameters
         OAEPParameterSpec     currentSpec;
     
         /**
-         * Return the PKCS#1 ASN.1 structure RSA-ES-OAEP-params.
+         * Return the PKCS#1 ASN.1 structure RSAES-OAEP-params.
          */
         protected byte[] engineGetEncoded() 
         {
-            ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
-            DEROutputStream         dOut = new DEROutputStream(bOut);
             AlgorithmIdentifier     hashAlgorithm = new AlgorithmIdentifier(
                                                             JCEDigestUtil.getOID(currentSpec.getDigestAlgorithm()),
                                                             new DERNull());
@@ -1145,15 +1078,12 @@ public abstract class JDKAlgorithmParameters
     
             try
             {
-                dOut.writeObject(oaepP);
-                dOut.close();
+                return oaepP.getEncoded(ASN1Encodable.DER);
             }
             catch (IOException e)
             {
                 throw new RuntimeException("Error encoding OAEPParameters");
             }
-    
-            return bOut.toByteArray();
         }
     
         protected byte[] engineGetEncoded(
@@ -1195,12 +1125,10 @@ public abstract class JDKAlgorithmParameters
             byte[] params) 
             throws IOException
         {
-            ASN1InputStream        aIn = new ASN1InputStream(params);
-    
             try
             {
-                RSAESOAEPparams oaepP = new RSAESOAEPparams((ASN1Sequence)aIn.readObject());
-    
+                RSAESOAEPparams oaepP = new RSAESOAEPparams((ASN1Sequence)ASN1Object.fromByteArray(params));
+
                 currentSpec = new OAEPParameterSpec(
                                        oaepP.getHashAlgorithm().getObjectId().getId(), 
                                        oaepP.getMaskGenAlgorithm().getObjectId().getId(), 
@@ -1245,7 +1173,7 @@ public abstract class JDKAlgorithmParameters
         PSSParameterSpec     currentSpec;
     
         /**
-         * Return the PKCS#1 ASN.1 structure RSA-ES-OAEP-params.
+         * Return the PKCS#1 ASN.1 structure RSASSA-PSS-params.
          */
         protected byte[] engineGetEncoded() 
             throws IOException
@@ -1304,12 +1232,10 @@ public abstract class JDKAlgorithmParameters
             byte[] params) 
             throws IOException
         {
-            ASN1InputStream        aIn = new ASN1InputStream(params);
-    
             try
             {
-                RSASSAPSSparams pssP = new RSASSAPSSparams((ASN1Sequence)aIn.readObject());
-    
+                RSASSAPSSparams pssP = new RSASSAPSSparams((ASN1Sequence)ASN1Object.fromByteArray(params));
+
                 currentSpec = new PSSParameterSpec(
                                        pssP.getHashAlgorithm().getObjectId().getId(), 
                                        pssP.getMaskGenAlgorithm().getObjectId().getId(), 
