@@ -12,7 +12,7 @@ import javax.mail.internet.MimeBodyPart;
 import javax.mail.internet.MimeMessage;
 import java.io.IOException;
 import java.security.NoSuchAlgorithmException;
-import java.security.NoSuchProviderException;
+import java.security.Provider;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.Map;
@@ -143,6 +143,8 @@ public class SMIMEGenerator
        
             content.setContent(message.getContent(), message.getContentType());
 
+            content.setDataHandler(message.getDataHandler());
+
             extractHeaders(content, message);
         }
         catch (MessagingException e)
@@ -161,7 +163,7 @@ public class SMIMEGenerator
         throws MessagingException
     {
         Enumeration e = message.getAllHeaders();
-
+        
         while (e.hasMoreElements())
         {
             Header hdr =(Header)e.nextElement();
@@ -172,8 +174,8 @@ public class SMIMEGenerator
 
     protected KeyGenerator createSymmetricKeyGenerator(
         String encryptionOID,
-        String provider)
-    throws NoSuchProviderException, NoSuchAlgorithmException
+        Provider provider)
+    throws NoSuchAlgorithmException
     {
         try
         {
@@ -203,8 +205,8 @@ public class SMIMEGenerator
 
     private KeyGenerator createKeyGenerator(
         String algName,
-        String provider)
-        throws NoSuchProviderException, NoSuchAlgorithmException
+        Provider provider)
+        throws NoSuchAlgorithmException
     {
         if (provider != null)
         {
