@@ -8,6 +8,7 @@ import org.bouncycastle.mail.smime.util.FileBackedMimeBodyPart;
 
 import javax.mail.BodyPart;
 import javax.mail.MessagingException;
+import javax.mail.Part;
 import javax.mail.internet.ContentType;
 import javax.mail.internet.MimeBodyPart;
 import javax.mail.internet.MimeMultipart;
@@ -17,8 +18,13 @@ import java.io.FilterOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.io.BufferedReader;
+import java.io.InputStreamReader;
 import java.security.cert.CertificateParsingException;
 import java.security.cert.X509Certificate;
+import java.security.Provider;
+import java.security.NoSuchProviderException;
+import java.security.Security;
 import java.util.Enumeration;
 
 public class SMIMEUtil
@@ -44,7 +50,25 @@ public class SMIMEUtil
 
         return !contentTransferEncoding.equalsIgnoreCase("binary");
     }
-    
+
+    public static Provider getProvider(String providerName)
+        throws NoSuchProviderException
+    {
+        if (providerName != null)
+        {
+            Provider prov = Security.getProvider(providerName);
+
+            if (prov != null)
+            {
+                return prov;
+            }
+
+            throw new NoSuchProviderException("provider " + providerName + " not found.");
+        }
+
+        return null;
+    }
+
     static class LineOutputStream extends FilterOutputStream
     {
         private static byte newline[];
@@ -143,6 +167,46 @@ public class SMIMEUtil
         }
     }
 
+    static void outputPostamble(LineOutputStream lOut, BodyPart parent, String parentBoundary, BodyPart part)
+        throws MessagingException, IOException
+    {
+        InputStream in;
+
+        try
+        {
+            in = ((MimeBodyPart)parent).getRawInputStream();
+        }
+        catch (MessagingException e)
+        {
+            return;   // no underlying content rely on default generation
+        }
+
+
+        MimeMultipart multipart = (MimeMultipart)part.getContent();
+        ContentType contentType = new ContentType(multipart.getContentType());
+        String boundary = "--" + contentType.getParameter("boundary");
+        int count = multipart.getCount() + 1;
+        String line;
+        while (count != 0 && (line = readLine(in)) != null)
+        {
+            if (line.startsWith(boundary))
+            {
+                count--;
+            }
+        }
+
+        while ((line = readLine(in)) != null)
+        {
+            if (line.startsWith(parentBoundary))
+            {
+                break;
+            }
+            lOut.writeln(line);
+        }
+
+        in.close();
+    }
+
     /*
      * read a line of input stripping of the tailing \r\n
      */
@@ -191,7 +255,8 @@ public class SMIMEUtil
                 Enumeration headers = mimePart.getAllHeaderLines();
                 while (headers.hasMoreElements())
                 {
-                    lOut.writeln((String)headers.nextElement());
+                    String header = (String)headers.nextElement();
+                    lOut.writeln(header);
                 }
 
                 lOut.writeln();      // CRLF separator
@@ -201,11 +266,20 @@ public class SMIMEUtil
                 for (int i = 0; i < mp.getCount(); i++)
                 {
                     lOut.writeln(boundary);
-                    outputBodyPart(out, mp.getBodyPart(i), defaultContentTransferEncoding);
-                    lOut.writeln();       // CRLF terminator
+                    BodyPart part = mp.getBodyPart(i);
+                    outputBodyPart(out, part, defaultContentTransferEncoding);
+                    if (!(part.getContent() instanceof MimeMultipart))
+                    {
+                        lOut.writeln();       // CRLF terminator needed
+                    }
+                    else
+                    {
+                        outputPostamble(lOut, mimePart, boundary, part);
+                    }
                 }
 
                 lOut.writeln(boundary + "--");
+
                 return;
             }
 
@@ -229,12 +303,27 @@ public class SMIMEUtil
                 out.flush();
                 return;
             }
-        
-            // 
-            // Special handling for Base64 or quoted-printable encoded
-            // body part - this is to get around JavaMail's habit of
-            // decoding and then re-encoding base64 data...
+
+            boolean base64 = contentTransferEncoding.equalsIgnoreCase("base64");
+
             //
+            // Write raw content, performing canonicalization
+            //
+            InputStream inRaw;
+
+            try
+            {
+                inRaw = mimePart.getRawInputStream();
+            }
+            catch (MessagingException e)
+            {
+                // this is less than ideal, but if the raw output stream is unavailable it's the
+                // best option we've got.
+                out = new CRLFOutputStream(out);
+                bodyPart.writeTo(out);
+                out.flush();
+                return;
+            }
 
             //
             // Write headers
@@ -242,23 +331,32 @@ public class SMIMEUtil
             LineOutputStream outLine = new LineOutputStream(out);
             for (Enumeration e = mimePart.getAllHeaderLines(); e.hasMoreElements();) 
             {
-                outLine.writeln((String)e.nextElement());
+                String header = (String)e.nextElement();
+  
+                outLine.writeln(header);
             }
 
             outLine.writeln();
             outLine.flush();
 
-            //
-            // Write raw content, performing canonicalization
-            //
-            InputStream inRaw = mimePart.getRawInputStream();
-            CRLFOutputStream outCRLF = new CRLFOutputStream(out);
+
+            OutputStream outCRLF;
+              
+            if (base64)
+            {
+                outCRLF = new Base64CRLFOutputStream(out);
+            }
+            else
+            {
+                outCRLF = new CRLFOutputStream(out);
+            }
 
             byte[]      buf = new byte[BUF_SIZE];
 
             int len;
             while ((len = inRaw.read(buf, 0, buf.length)) > 0)
             {
+
                 outCRLF.write(buf, 0, len);
             }
 
@@ -405,4 +503,74 @@ public class SMIMEUtil
             this.dispose();
         }
     }
+
+    static class Base64CRLFOutputStream extends FilterOutputStream
+    {
+        protected int lastb;
+        protected static byte newline[];
+        private boolean isCrlfStream;
+
+        public Base64CRLFOutputStream(OutputStream outputstream)
+        {
+            super(outputstream);
+            lastb = -1;
+        }
+
+        public void write(int i)
+            throws IOException
+        {
+            if (i == '\r')
+            {
+                out.write(newline);
+            }
+            else if (i == '\n')
+            {
+                if (lastb != '\r')
+                {                                 // imagine my joy...
+                    if (!(isCrlfStream && lastb == '\n'))
+                    {
+                        out.write(newline);
+                    }
+                }
+                else
+                {
+                    isCrlfStream = true;
+                }
+            }
+            else
+            {
+                out.write(i);
+            }
+
+            lastb = i;
+        }
+
+        public void write(byte[] buf)
+            throws IOException
+        {
+            this.write(buf, 0, buf.length);
+        }
+
+        public void write(byte buf[], int off, int len)
+            throws IOException
+        {
+            for (int i = off; i != off + len; i++)
+            {
+                this.write(buf[i]);
+            }
+        }
+
+        public void writeln()
+            throws IOException
+        {
+            super.out.write(newline);
+        }
+
+        static
+        {
+            newline = new byte[2];
+            newline[0] = '\r';
+            newline[1] = '\n';
+        }
+    }
 }
