@@ -2,6 +2,9 @@ package org.bouncycastle.openpgp;
 
 import org.bouncycastle.bcpg.SignatureSubpacket;
 import org.bouncycastle.bcpg.SignatureSubpacketTags;
+import org.bouncycastle.bcpg.BCPGInputStream;
+import org.bouncycastle.bcpg.SignaturePacket;
+import org.bouncycastle.bcpg.BCPGOutputStream;
 import org.bouncycastle.bcpg.sig.Exportable;
 import org.bouncycastle.bcpg.sig.KeyExpirationTime;
 import org.bouncycastle.bcpg.sig.KeyFlags;
@@ -13,10 +16,14 @@ import org.bouncycastle.bcpg.sig.SignatureCreationTime;
 import org.bouncycastle.bcpg.sig.SignatureExpirationTime;
 import org.bouncycastle.bcpg.sig.SignerUserID;
 import org.bouncycastle.bcpg.sig.TrustSignature;
+import org.bouncycastle.bcpg.sig.EmbeddedSignature;
 
 import java.util.ArrayList;
 import java.util.Date;
 import java.util.List;
+import java.io.IOException;
+import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
 
 /**
  * Generator for signature subpackets.
@@ -140,7 +147,29 @@ public class PGPSignatureSubpacketGenerator
         
         list.add(new SignerUserID(isCritical, userID));
     }
-    
+
+    public void setEmbeddedSignature(
+        boolean isCritical,
+        PGPSignature pgpSignature)
+        throws IOException
+    {
+        byte[] sig = pgpSignature.getEncoded();
+        byte[] data;
+
+        if (sig.length - 1 > 256)
+        {
+            data = new byte[sig.length - 3];
+        }
+        else
+        {
+            data = new byte[sig.length - 2];
+        }
+
+        System.arraycopy(sig, sig.length - data.length, data, 0, data.length);
+        
+        list.add(new EmbeddedSignature(isCritical, data));
+    }
+
     public void setPrimaryUserID(
         boolean     isCritical,
         boolean     isPrimaryUserID)
